package java.awt;

import java.awt.event.MouseWheelEvent;
import sun.awt.SunToolkit;
import sun.awt.AppContext;
import sun.awt.AWTAccessor;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import sun.awt.dnd.SunDropTargetEvent;
import java.lang.ref.WeakReference;
import sun.util.logging.PlatformLogger;
import java.awt.event.AWTEventListener;
import java.io.Serializable;

class LightweightDispatcher implements Serializable, AWTEventListener
{
    private static final long serialVersionUID = 5184291520170872969L;
    private static final int LWD_MOUSE_DRAGGED_OVER = 1500;
    private static final PlatformLogger eventLog;
    private static final int BUTTONS_DOWN_MASK;
    private Container nativeContainer;
    private Component focus;
    private transient WeakReference<Component> mouseEventTarget;
    private transient WeakReference<Component> targetLastEntered;
    private transient WeakReference<Component> targetLastEnteredDT;
    private transient boolean isMouseInNativeContainer;
    private transient boolean isMouseDTInNativeContainer;
    private Cursor nativeCursor;
    private long eventMask;
    private static final long PROXY_EVENT_MASK = 131132L;
    private static final long MOUSE_MASK = 131120L;
    
    LightweightDispatcher(Container nativeContainer) {
        this.isMouseInNativeContainer = false;
        this.isMouseDTInNativeContainer = false;
        this.nativeContainer = nativeContainer;
        this.mouseEventTarget = new WeakReference(null);
        this.targetLastEntered = new WeakReference(null);
        this.targetLastEnteredDT = new WeakReference(null);
        this.eventMask = 0L;
    }
    
    void dispose() {
        this.stopListeningForOtherDrags();
        this.mouseEventTarget.clear();
        this.targetLastEntered.clear();
        this.targetLastEnteredDT.clear();
    }
    
    void enableEvents(long n) {
        this.eventMask |= n;
    }
    
    boolean dispatchEvent(AWTEvent awtEvent) {
        boolean n = false;
        if ((awtEvent instanceof SunDropTargetEvent)) {
            n = this.processDropTargetEvent((SunDropTargetEvent)awtEvent);
        }
        else {
            if (awtEvent instanceof MouseEvent && (this.eventMask & 0x20030L) != 0x0L) {
                n = this.processMouseEvent((MouseEvent)awtEvent);
            }
            if (awtEvent.getID() == 503) {
                this.nativeContainer.updateCursorImmediately();
            }
        }
        return n;
    }
    
    private boolean isMouseGrab(MouseEvent mouseEvent) {
        int modifiersEx = mouseEvent.getModifiersEx();
        if (mouseEvent.getID() == 501 || mouseEvent.getID() == 502) {
            modifiersEx ^= InputEvent.getMaskForButton(mouseEvent.getButton());
        }
        return (modifiersEx & LightweightDispatcher.BUTTONS_DOWN_MASK) != 0x0;
    }
    
    private boolean processMouseEvent(MouseEvent mouseEvent) {
        int id = mouseEvent.getID();
        Component mouseEventTarget = this.nativeContainer.getMouseEventTarget(mouseEvent.getX(), mouseEvent.getY(), true);
        this.trackMouseEnterExit(mouseEventTarget, mouseEvent);
        Component referent = (Component)this.mouseEventTarget.get();
        if (!this.isMouseGrab(mouseEvent) && id != 500) {
            referent = ((mouseEventTarget != this.nativeContainer) ? mouseEventTarget : null);
            this.mouseEventTarget = new WeakReference(referent);
        }
        if (referent != null) {
            switch (id) {
                case 504:
                case 505: {
                    break;
                }
                case 501: {
                    this.retargetMouseEvent(referent, id, mouseEvent);
                    break;
                }
                case 502: {
                    this.retargetMouseEvent(referent, id, mouseEvent);
                    break;
                }
                case 500: {
                    if (mouseEventTarget == referent) {
                        this.retargetMouseEvent(mouseEventTarget, id, mouseEvent);
                        break;
                    }
                    break;
                }
                case 503: {
                    this.retargetMouseEvent(referent, id, mouseEvent);
                    break;
                }
                case 506: {
                    if (this.isMouseGrab(mouseEvent)) {
                        this.retargetMouseEvent(referent, id, mouseEvent);
                        break;
                    }
                    break;
                }
                case 507: {
                    if (LightweightDispatcher.eventLog.isLoggable(PlatformLogger.Level.FINEST) && mouseEventTarget != null) {
                        LightweightDispatcher.eventLog.finest("retargeting mouse wheel to " + mouseEventTarget.getName() + ", " + mouseEventTarget.getClass());
                    }
                    this.retargetMouseEvent(mouseEventTarget, id, mouseEvent);
                    break;
                }
            }
            if (id != 507) {
                mouseEvent.consume();
            }
        }
        return mouseEvent.isConsumed();
    }
    
    private boolean processDropTargetEvent(SunDropTargetEvent sunDropTargetEvent) {
        int id = sunDropTargetEvent.getID();
        int x = sunDropTargetEvent.getX();
        int y = sunDropTargetEvent.getY();
        if (!this.nativeContainer.contains(x, y)) {
            Dimension size = this.nativeContainer.getSize();
            if (size.width <= x) {
                x = size.width - 1;
            }
            else if (x < 0) {
                x = 0;
            }
            if (size.height <= y) {
                y = size.height - 1;
            }
            else if (y < 0) {
                y = 0;
            }
        }
        Component dropTargetEventTarget = this.nativeContainer.getDropTargetEventTarget(x, y, true);
        this.trackMouseEnterExit(dropTargetEventTarget, sunDropTargetEvent);
        if (dropTargetEventTarget != this.nativeContainer && dropTargetEventTarget != null) {
            switch (id) {
                case 504:
                case 505: {
                    break;
                }
                default: {
                    this.retargetMouseEvent(dropTargetEventTarget, id, sunDropTargetEvent);
                    sunDropTargetEvent.consume();
                    break;
                }
            }
        }
        return sunDropTargetEvent.isConsumed();
    }
    
    private void trackDropTargetEnterExit(Component component, MouseEvent mouseEvent) {
        int id = mouseEvent.getID();
        if (id == 504 && this.isMouseDTInNativeContainer) {
            this.targetLastEnteredDT.clear();
        }
        else if (id == 504) {
            this.isMouseDTInNativeContainer = true;
        }
        else if (id == 505) {
            this.isMouseDTInNativeContainer = false;
        }
        this.targetLastEnteredDT = new WeakReference(this.retargetMouseEnterExit(component, mouseEvent, (Component)this.targetLastEnteredDT.get(), this.isMouseDTInNativeContainer));
    }
    
    private void trackMouseEnterExit(Component component, MouseEvent mouseEvent) {
        if ((mouseEvent instanceof SunDropTargetEvent)) {
            this.trackDropTargetEnterExit(component, mouseEvent);
            return;
        }
        int id = mouseEvent.getID();
        if (id != 505 && id != 506 && id != 1500 && !this.isMouseInNativeContainer) {
            this.isMouseInNativeContainer = true;
            this.startListeningForOtherDrags();
        }
        else if (id == 505) {
            this.isMouseInNativeContainer = false;
            this.stopListeningForOtherDrags();
        }
        this.targetLastEntered = new WeakReference(this.retargetMouseEnterExit(component, mouseEvent, (Component)this.targetLastEntered.get(), this.isMouseInNativeContainer));
        return;
    }
    
    private Component retargetMouseEnterExit(Component component, MouseEvent mouseEvent, Component component2, boolean b) {
        int id = mouseEvent.getID();
        Component component3 = b ? component : null;
        if (component2 != component3) {
            if (component2 != null) {
                this.retargetMouseEvent(component2, 505, mouseEvent);
            }
            if (id == 505) {
                mouseEvent.consume();
            }
            if (component3 != null) {
                this.retargetMouseEvent(component3, 504, mouseEvent);
            }
            if (id == 504) {
                mouseEvent.consume();
            }
        }
        return component3;
    }
    
    private void startListeningForOtherDrags() {
        AccessController.doPrivileged(new PrivilegedAction<Object>() {
            @Override
            public Object run() {
                LightweightDispatcher.this.nativeContainer.getToolkit().addAWTEventListener(LightweightDispatcher.this, 48L);
                return null;
            }
        });
    }
    
    private void stopListeningForOtherDrags() {
        AccessController.doPrivileged(new PrivilegedAction<Object>() {
            @Override
            public Object run() {
                LightweightDispatcher.this.nativeContainer.getToolkit().removeAWTEventListener(LightweightDispatcher.this);
                return null;
            }
        });
    }
    
    @Override
    public void eventDispatched(AWTEvent awtEvent) {
        if (!(awtEvent instanceof MouseEvent) || awtEvent instanceof SunDropTargetEvent || awtEvent.id != 506 || awtEvent.getSource() == this.nativeContainer) {
            return;
        }
        MouseEvent mouseEvent = (MouseEvent)awtEvent;
        final Object o;
        synchronized (this.nativeContainer.getTreeLock()) {
            Component component = mouseEvent.getComponent();
            if (!component.isShowing()) {
                return;
            }
            Container container;
            for (container = this.nativeContainer; container != null && !(container instanceof Window); container = container.getParent_NoClientCode()) {}
            if (container == null || ((Window)container).isModalBlocked()) {
                return;
            }
            o = new MouseEvent(this.nativeContainer, 1500, mouseEvent.getWhen(), mouseEvent.getModifiersEx() | mouseEvent.getModifiers(), mouseEvent.getX(), mouseEvent.getY(), mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), mouseEvent.getButton());
            AWTAccessor.MouseEventAccessor mouseEventAccessor = AWTAccessor.getMouseEventAccessor();
            mouseEventAccessor.setCausedByTouchEvent(o, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
            mouseEvent.copyPrivateDataInto(o);
            final Point locationOnScreen = component.getLocationOnScreen();
            if (AppContext.getAppContext() != this.nativeContainer.appContext) {
                SunToolkit.executeOnEventHandlerThread(this.nativeContainer, new Runnable() {
                    @Override
                    public void run() {
                        if (!LightweightDispatcher.this.nativeContainer.isShowing()) {
                            return;
                        }
                        Point locationOnScreen = LightweightDispatcher.this.nativeContainer.getLocationOnScreen();
                        o.translatePoint(locationOnScreen.x - locationOnScreen.x, locationOnScreen.y - locationOnScreen.y);
                        LightweightDispatcher.access$100(LightweightDispatcher.this, LightweightDispatcher.this.nativeContainer.getMouseEventTarget(o.getX(), o.getY(), true), o);
                        return;
                    }
                });
                return;
            }
            if (!this.nativeContainer.isShowing()) {
                return;
            }
            Point locationOnScreen2 = this.nativeContainer.getLocationOnScreen();
            o.translatePoint(locationOnScreen.x - locationOnScreen2.x, locationOnScreen.y - locationOnScreen2.y);
        }
        this.trackMouseEnterExit(this.nativeContainer.getMouseEventTarget(o.getX(), o.getY(), true), o);
        return;
    }
    
    void retargetMouseEvent(Component parent, int id, MouseEvent mouseEvent) {
        if (parent == null) {
            return;
        }
        int x = mouseEvent.getX();
        int y = mouseEvent.getY();
        for (; parent != null && parent != this.nativeContainer; parent = parent.getParent()) {
            x -= parent.x;
            y -= parent.y;
        }
        if (parent != null) {
            Object o;
            if ((mouseEvent instanceof SunDropTargetEvent)) {
                o = new SunDropTargetEvent(parent, id, x, y, ((SunDropTargetEvent)mouseEvent).getDispatcher());
            }
            else if (id == 507) {
                o = new MouseWheelEvent(parent, id, mouseEvent.getWhen(), mouseEvent.getModifiersEx() | mouseEvent.getModifiers(), x, y, mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), ((MouseWheelEvent)mouseEvent).getScrollType(), ((MouseWheelEvent)mouseEvent).getScrollAmount(), ((MouseWheelEvent)mouseEvent).getWheelRotation(), ((MouseWheelEvent)mouseEvent).getPreciseWheelRotation());
            }
            else {
                o = new MouseEvent(parent, id, mouseEvent.getWhen(), mouseEvent.getModifiersEx() | mouseEvent.getModifiers(), x, y, mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), mouseEvent.getButton());
                AWTAccessor.MouseEventAccessor mouseEventAccessor = AWTAccessor.getMouseEventAccessor();
                mouseEventAccessor.setCausedByTouchEvent(o, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
            }
            mouseEvent.copyPrivateDataInto(o);
            if (parent == this.nativeContainer) {
                ((Container)parent).dispatchEventToSelf(o);
            }
            else {
                assert AppContext.getAppContext() == parent.appContext;
                if (this.nativeContainer.modalComp != null) {
                    if (((Container)this.nativeContainer.modalComp).isAncestorOf(parent)) {
                        parent.dispatchEvent(o);
                    }
                    else {
                        mouseEvent.consume();
                    }
                }
                else {
                    parent.dispatchEvent(o);
                }
            }
            if (id == 507 && o.isConsumed()) {
                mouseEvent.consume();
            }
        }
        return;
    }
    
    static void access$100(LightweightDispatcher lightweightDispatcher, Component component, MouseEvent mouseEvent) {
        lightweightDispatcher.trackMouseEnterExit(component, mouseEvent);
    }
    
    static {
        eventLog = PlatformLogger.getLogger("java.awt.event.LightweightDispatcher");
        int[] buttonDownMasks = AWTAccessor.getInputEventAccessor().getButtonDownMasks();
        int buttons_DOWN_MASK = 0;
        for (int length = buttonDownMasks.length, i = 0; i < length; ++i) {
            buttons_DOWN_MASK |= buttonDownMasks[i];
        }
        BUTTONS_DOWN_MASK = buttons_DOWN_MASK;
    }
}
